{% set label = "<p><b><i>" ~ field_name ~ '</i></b>&nbsp(type:&nbsp' ~ data_type ~ ")</p>" %}





<div class="flex flex-row justify-start gap-2">
<input type="checkbox" id="{{ element_id }}--input-field" is_input="true" name="{{ field_name }}" value="true">

<label for="{{ element_id }}--input-field">{{ label | safe }}</label>
</div>
