<div class="w-full grid grid-cols-1 justify-items-center p-8">
{{ macros.value_selectbox(element_id="value_select_1", data_types=[], show_data_type=True, allow_preview=True, label="Select value", field_name="preview") }}
</div>
