<div class="w-full grid grid-cols-1 justify-items-stretch p-8 gap-4">

{{ macros.operation_run_panel(element_id="operation", operation_id="table.cut_column") }}
{#{{ macros.operation_run_panel(element_id="operation") }}#}

</div>
