@tailwind base;
@tailwind components;
@tailwind utilities;

@layer base {
    table {
        @apply border-collapse border border-slate-400 w-full p-4;
    }
    tr {
        @apply odd:bg-white even:bg-slate-50
    }
    th {
        @apply border border-slate-300;
    }
    td {
        @apply border border-slate-300 py-1 px-2 truncate max-w-xs;
    }
}
