{% extends "kiara_plugin.service/base.html" %}

{% block content %}
<h2>Operations</h2>
{% include'kiara_plugin.service/operations/index.html' %}
<h2>Values</h2>
{% include'kiara_plugin.service/values/index.html' %}

{% endblock %}
