<div class="grid grid-cols-2 gap-4 items-start">
    {% for field_name, rendered in fields.items() %}
        <div class="bg-blue-200 rounded-lg p-4 col-span-1">
        {{ rendered | safe }}
        </div>
    {%  endfor %}
</div>
